package mschmidt.komo.datatools;

/*
 * Instances of this object class are used to obtain database information.
 * This task is performed in a helper thread.
 */

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.SQLException;
import java.sql.Types;

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationActionBarAdvisor;
import mschmidt.komo.DbInfoDialog;
import mschmidt.komo.ApplicationImages;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.preferences.PreferenceConstants;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.progress.IProgressConstants;

public class DatabaseInfoJob extends Job {

	public DatabaseInfoJob(final String jobName) {
		super(jobName);
	}

	@Override
	protected final IStatus run(final IProgressMonitor monitor) {
		// Initialize a boolean variable to track the status of the job.
		boolean ranOK = true;
		// Initialize a string buffer to collect the information.
		StringBuffer sbInfo = new StringBuffer(
				"*** Database Information:\n\n*** Database Program Details:\n");
		// Initialize callable statement
		CallableStatement callStmt = null;
		try {
			setProperty(IProgressConstants.ICON_PROPERTY, KomoPlugin
					.getImageDescriptor(ApplicationImages.DB_ICON));
			monitor.beginTask("Database Info", 400);
			IPreferenceStore store = KomoPlugin.getDefault()
					.getPreferenceStore();
			String stTemp1, stTemp2;
			Application.dbConn.dbLock.acquire();
			Connection thisConnection = Application.dbConn.getConnection();
			Driver thisDriver = Application.dbConn.getDriver();
			if (thisConnection == null) {
				sbInfo.append("No information available because there is no database connection.\nReview connection settings and try again.");
				ranOK = false;
				return Status.CANCEL_STATUS;
			}
			monitor.worked(100);
			try {
				DatabaseMetaData dbMeta = thisConnection.getMetaData();
				sbInfo.append("Database Program:  "
						+ dbMeta.getDatabaseProductName()
						+ "\nVersion:  "
						+ dbMeta.getDatabaseProductVersion()
						+ "\n*** SQL Support Details:\nANSI 92 Entry level:  "
						+ dbMeta.supportsANSI92EntryLevelSQL()
						+ "\nANSI 92 Intermediate level:  "
						+ dbMeta.supportsANSI92IntermediateSQL()
						+ "\nANSI 92 Full level:  "
						+ dbMeta.supportsANSI92FullSQL()
						+ "\nCore SQL Grammar:  "
						+ dbMeta.supportsCoreSQLGrammar()
						+ "\nExtended SQL Grammar:  "
						+ dbMeta.supportsExtendedSQLGrammar()
						+ "\n");
				stTemp2 = "Driver Name:  " + dbMeta.getDriverName()
						+ "\nVersion:  " + dbMeta.getDriverVersion() + "\n";
			} catch (Exception e) {
				ranOK = false;
				sbInfo.append(e.getMessage()
					+ "\nDatabase Program:  Unavailable\nVersion:  Unavailable\n");
				stTemp2 = "Driver Name:  Unavailable\nVersion:  Unavailable\n*** SQL Support\nUnavailable\n";
				e.printStackTrace();
			}
			monitor.worked(100);
			try {
				callStmt = thisConnection
					.prepareCall("{ ? = call pg_postmaster_start_time()}");
				callStmt.registerOutParameter(1, Types.TIMESTAMP);
				callStmt.execute();
				stTemp1 = callStmt.getTimestamp(1).toString();
				sbInfo.append("Postmaster Started:  " + stTemp1 + "\n");
			} catch (Exception e) {
				ranOK = false;
				sbInfo.append(e.getMessage()
						+ "\nPostmaster Started:  Unavailable\n");
				e.printStackTrace();
			}
			stTemp1 = store.getString(PreferenceConstants.JDBC_PATH);
			if (stTemp1 == null || stTemp1.length() == 0) {
				sbInfo.append("*** JDBC Details:\nPath to JDBC:  Not Available because it has not been set.\n"
							+ stTemp2);
			} else {
				sbInfo.append("*** JDBC Details:\nPath to JDBC:  "
						+ stTemp1 + "\n" + stTemp2);
			}

			if (thisDriver == null) {
				stTemp1 = "Fully JDBC Compliant:  Unavailable\n";
			} else {
				stTemp1 = "Fully JDBC Compliant:  "
						+ thisDriver.jdbcCompliant() + "\n";
			}
			sbInfo.append(stTemp1);

			sbInfo.append("\n*** Connection Details:\n");
			stTemp1 = store.getString(PreferenceConstants.DB_URL);
			if (stTemp1 == null || stTemp1.length() == 0) {
				sbInfo.append("Host Address:  Unavailable\n");
			} else {
				sbInfo.append("Host Address:  " + stTemp1 + "\n");
			}

			stTemp1 = store.getString(PreferenceConstants.DB_PORT);
			if (stTemp1 == null || stTemp1.length() == 0) {
				sbInfo.append("Host Port:  Unavailable\n");
			} else {
				sbInfo.append("Host Port:  " + stTemp1 + "\n");
			}

			sbInfo.append("Secure Socket Layer Enabled:  "
					+ store.getBoolean(PreferenceConstants.DB_SSL) + "\n");

			sbInfo.append("\n*** User Details:\nName:  ");
			stTemp1 = store.getString(PreferenceConstants.DB_USER);
			if (stTemp1 == null || stTemp1.length() == 0) {
				sbInfo.append("Unavailable\nPassword:  ********\n");
			} else {
				sbInfo.append(stTemp1 + "\nPassword:  ********\n");
			}

			monitor.worked(100);
			try {
				callStmt = thisConnection
					.prepareCall("{ ? = call has_database_privilege( ? , ? )}");
				callStmt.registerOutParameter(1, Types.BIT);
				callStmt.setString(2, store
						.getString(PreferenceConstants.DB_NAME));
				callStmt.setString(3, "CREATE");
				boolean canCreate, canTemp;
				callStmt.execute();
				canCreate = callStmt.getBoolean(1);
				callStmt.setString(3, "TEMPORARY");
				callStmt.execute();
				canTemp = callStmt.getBoolean(1);
				sbInfo.append("*** Privileges Details:\nDatabase Creation:  "
					+ canCreate
					+ "\nTemporary Table:  "
					+ canTemp + "\n");
			} catch (Exception e) {
				ranOK = false;
				sbInfo.append(e.getMessage()
					+ "\n*** Privileges Details:\nDatabase Creation:  Unavailable\nTemporary Table:  Unavailable\n");
				e.printStackTrace();
			}
			sbInfo.append("\n*** Database Details:\n");
			stTemp1 = store.getString(PreferenceConstants.DB_NAME);
			if (stTemp1 == null || stTemp1.length() == 0) {
				sbInfo.append("Name:  Unavailable\n");
			} else {
				sbInfo.append("Name:  " + stTemp1 + "\n");
			}

			monitor.worked(100);
			System.out.println("Got this far");
			try {
				callStmt = thisConnection
						.prepareCall("{ ? = call pg_database_size( ? )}");
				callStmt.registerOutParameter(1, Types.BIGINT);
				callStmt.setString(2, store
						.getString(PreferenceConstants.DB_NAME));
				callStmt.execute();
				long sizeInt = callStmt.getLong(1);
				callStmt = thisConnection
						.prepareCall("{ ? = call pg_size_pretty( ? )}");
				callStmt.registerOutParameter(1, Types.VARCHAR);
				callStmt.setLong(2, sizeInt);
				callStmt.execute();
				stTemp1 = callStmt.getString(1);
				sbInfo.append("Size:  " + stTemp1 + "\n");
			} catch (Exception e) {
				ranOK = false;
				sbInfo.append(e.getMessage() + "\nSize:  Unavailable\n");
				e.printStackTrace();
			}

		} catch (Exception e) {
			sbInfo.append(e.getMessage());
			ranOK = false;
			e.printStackTrace();
		} finally {
			// Everything is finished, so end the monitor,
			// close the connection, and release the lock.
			try {
				if (callStmt != null) {
					callStmt.close();
				}
			} catch (SQLException e) {
				sbInfo.append(e.getMessage() + "\n");
				e.printStackTrace();
			}
			monitor.done();
			Application.dbConn.closeConnection();
			Application.dbConn.dbLock.release();
			showResults(sbInfo.toString(), ranOK);
		}
		return Status.OK_STATUS;
	}

	/**
	 * This method writes the results to a dialog.
	 * 
	 * @param message -
	 *            the information text for the dialog.
	 */
	protected final void showResults(final String message, final boolean ranOK) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if (!ranOK) {
					String dbMsg = Application.dbConn.getMessage();
					ApplicationActionBarAdvisor.postMessage(dbMsg, true);
				}
				DbInfoDialog dbDialog = new DbInfoDialog(Display.getDefault()
					.getActiveShell(), message);
				dbDialog.create();
				dbDialog.getShell().setText("Database Information");
				dbDialog.getShell().setImage(
					KomoPlugin.getImageDescriptor(ApplicationImages.DB_ICON)
							.createImage());
				dbDialog.open();
			}
		});
	}

}
